@@ -733,7 +733,7 @@ class BIRDConfigBuilder(ConfigBuilder):
     IGNORABLE_ISSUES = ["max_prefix_count_rejected_routes"]
 
     AVAILABLE_VERSION = ["1.6.3", "1.6.4", "1.6.6", "1.6.7", "1.6.8",
-                         "2.0.7"]
+                         "2.0.7", "2.0.7+b962967e"]
     DEFAULT_VERSION = "1.6.8"
 
     def validate_bgpspeaker_specific_configuration(self):
@@ -751,10 +751,20 @@ class BIRDConfigBuilder(ConfigBuilder):
         if hooks:
             if not isinstance(hooks, list):
                 raise BuilderError(
-                    "hooks must be a list of hook names"
+                    "hooks must be a list of hook names."
                 )
 
-        if version.parse(self.target_version) >= version.parse("2.0.7"):
+        for client in self.cfg_clients.cfg["clients"]:
+            if client["cfg"]["multihop"]:
+                if self.cfg_general["path_hiding"]:
+                    raise BuilderError(
+                        "multihop is not supported on BIRD configurations "
+                        "when path_hiding mitigation is enabled; "
+                        "see https://github.com/pierky/arouteserver/pull/61 "
+                        "for more details."
+                    )
+
+        if version.parse(self.target_version) == version.parse("2.0.7"):
             max_prefix_count_rejected_routes_clients = []
 
             for client in self.cfg_clients.cfg["clients"]:
@@ -780,11 +790,12 @@ class BIRDConfigBuilder(ConfigBuilder):
                     "default value of 'count_rejected_routes' is True, "
                     "it's not advisable to deploy any configuration for "
                     "BIRD 2.0.7 that uses that setting at the moment. "
-                    "Now there are two ways to unblock the build of the "
+                    "Now, there are 3 ways to unblock the build of the "
                     "configuration: 1) this error can be ignored using "
                     "the command line argument mentioned later on in "
-                    "this message, or 2) the value of "
-                    "'count_rejected_routes' can be set to False. "
+                    "this message; 2) the value of "
+                    "'count_rejected_routes' can be set to False; 3) "
+                    "--target-version can be set to 2.0.7+b962967e. "
                     "If this error will be ignored, the configuration "
                     "will be generated as if that option was set to "
                     "False; this may be a good option if the goal is "
@@ -803,11 +814,16 @@ class BIRDConfigBuilder(ConfigBuilder):
                     "configurations generated by this tool. For more "
                     "details on how to configure it, see the general.yml "
                     "file distributed with the package or check this "
-                    "URL: {link_ars}".format(
+                    "URL: {link_ars}. The third option is to apply the "
+                    "patch mentioned at {patch_url} and pass the value "
+                    "2.0.7+b962967e via the --target-version command "
+                    "line argument.".format(
                         link_bird=("https://www.mail-archive.com/bird-users"
                                    "@network.cz/msg05597.html"),
                         link_ars=("https://arouteserver.readthedocs.io/"
-                                  "en/latest/GENERAL.html#max-prefix-max-prefix")
+                                  "en/latest/GENERAL.html#max-prefix-max-prefix"),
+                        patch_url=("https://www.mail-archive.com/bird-users"
+                                   "@network.cz/msg05638.html")
                     )
                 ):
                     res = False
@@ -850,8 +866,9 @@ class OpenBGPDConfigBuilder(ConfigBuilder):
                        "footer"]
     LOCAL_FILES_BASE_DIR = "/etc/bgpd"
 
-    AVAILABLE_VERSION = ["6.0", "6.1", "6.2", "6.3", "6.4", "6.5", "6.6", "6.7"]
-    DEFAULT_VERSION = "6.6"
+    AVAILABLE_VERSION = ["6.0", "6.1", "6.2", "6.3", "6.4", "6.5", "6.6", "6.7",
+                         "6.8"]
+    DEFAULT_VERSION = "6.7"
 
     IGNORABLE_ISSUES = ["path_hiding", "transit_free_action",
                         "add_path", "max_prefix_action",
